% TropD EDJ metric
% Latitude of the eddy driven jet (EDJ)
% Written by Ori Adam Mar.20.2017
% Methods:
% peak {default}: Latitude of the maximum of the zonal wind at the level closest to the 850 hPa level (smoothing parameter n=30)
% max: Latitude of the maximum of the zonal wind at the level closest to the 850 hPa level (smoothing parameter n=6)
% fit: Latitude of the maximum of the zonal wind at the level closest to the 850 hPa level using a quadratic polynomial fit of data from gridpoints surrounding the gridpoint of the maximum
%
% Syntax:
% for the 'peak' method
% >> [PhiSH PhiNH] = TropD_Metric_EDJ(U,lat,lev,'peak')
% for the 'max' method
% >> [PhiSH PhiNH] = TropD_Metric_EDJ(U,lat,lev,'max')
% or
% >> [PhiSH PhiNH] = TropD_Metric_EDJ(U,lat,lev,'max',n)
% where n>=1 is the smoothing parameter (default value = 6)
% for the 'fit' method
% >> [PhiSH PhiNH] = TropD_Metric_EDJ(U,lat,lev,'fit')
% or
% >> [PhiSH PhiNH] = TropD_Metric_EDJ(U,lat,lev,'fit',m)
% where m=1,2,3,... (default = 1) is the number of points on either side of the gridpoint of the maximum, used in the polynomial fit. 
%
% Input:
% U(lat,lev) = zonal mean zonal wind. Surface wind or uni-level zonal wind data can also be used as an input field using U(lat,1)
% lat = equally-spaced latitude column vector
% lev = vertical pressure-level vector [hPa]
% method (optional) = 'peak' {default} |  'max'  | 'fit'
% Output:
% PhiSH = latitude of the EDJ metric in the SH
% PhiNH = latitude of the EDJ metric in the NH


function [PhiSH , PhiNH] = TropD_Metric_EDJ(U,lat,lev,method,n)

if nargin < 4
    method = 'peak' ;
end
if nargin == 5
    if  ~isscalar(n) || n<1
        if ismember(method,{'max','peak'})
            disp('TropD_Metric_EDJ: ERROR : the smoothing parameter n must be >= 1' ) ;
        else
            disp('TropD_Metric_EDJ: ERROR : m, the number of gridpoints on either side of the maximum used in the polynomial expansion, must be 1,2,3...' ) ;
        end
        PhiNH = [] ;
        PhiSH = [] ;
        return
    end
else
    n = 0 ; % Flag for using default method values
end
eq_boundary    = 15 ;
polar_boundary = 70 ;
lev = double(lev) ;
try
    if ~iscolumn(lev)
        u = U(:,dsearchn(lev',850)) ;
    else
        u = U(:,dsearchn(lev,850)) ;
    end
catch % in case dsearchn is not supported
    u = U(:,find(lev==850)) ;
end
switch method
    case 'max'
        if n
            PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary),n) ;
            PhiSH = TropD_Calculate_MaxLat(u(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary),n) ;
        else
            PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary)) ;
            PhiSH = TropD_Calculate_MaxLat(u(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary)) ;
        end
    case 'peak'
        if n
            PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary),n) ; %
            PhiSH = TropD_Calculate_MaxLat(u(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary),n) ;
        else
            PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary),30) ; % for n>=30, result converges to max(F)
            PhiSH = TropD_Calculate_MaxLat(u(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary),30) ;
        end
    case 'fit'
        if ~n  % determine number of gridpoints on either side of the peak to be used in the 2nd order polynomial fit
            n = 1 ; % default
        end
        % NH
        Uh = u(lat>eq_boundary & lat < polar_boundary) ;
        Lat =lat(lat>eq_boundary & lat < polar_boundary) ;
        % Find gridpont of max value
        try
            [m Im] = nanmax(Uh) ;
        catch % in case nanmax is not suppoerted
            [m Im] = max(Uh) ;
        end
        if Im == 1 || Im == length(Uh)
            PhiNH = Lat(Im) ;
        elseif n> Im-1  || n > length(Uh) - Im  % use largest allowable n
            N = min(Im-1,length(Uh) - Im) ;
            p = polyfit(Lat(Im-N:Im+N),Uh(Im-N:Im+N),2) ;
            PhiNH = -p(2)/(2*p(1)) ;
        else
            p = polyfit(Lat(Im-n:Im+n),Uh(Im-n:Im+n),2) ;
            PhiNH = -p(2)/(2*p(1)) ;
        end
        
        % QA
        %figure ;
        %plot(Lat,Uh,'-',Lat(Im),m,'*r',Lat(Im-n:Im+n),polyval(p,Lat(Im-n:Im+n)),'-b',Lat(Im+n):0.01:Lat(Im-n),polyval(p,Lat(Im+n):0.01:Lat(Im-n)),'-g') ;  hold on
        %plot(PhiNH,max(polyval(p,Lat(Im+n):0.01:Lat(Im-n))),'xb') ;  hold on
        % SH
        Uh = u(lat>-polar_boundary & lat < -eq_boundary) ;
        Lat =lat(lat>-polar_boundary & lat < -eq_boundary) ;
        % Find gridpont of max value
        try
            [m Im] = nanmax(Uh) ;
        catch % in case nanmax is not suppoerted
            [m Im] = max(Uh) ;
        end
        if Im == 1 || Im == length(Uh)
            PhiSH = Lat(Im) ;
        elseif n> Im-1  || n > length(Uh) - Im  % use largest allowable n
            N = min(Im-1,length(Uh) - Im) ;
            p = polyfit(Lat(Im-N:Im+N),Uh(Im-N:Im+N),2) ;
            PhiSH = -p(2)/(2*p(1)) ;
        else
            p = polyfit(Lat(Im-n:Im+n),Uh(Im-n:Im+n),2) ;
            PhiSH = -p(2)/(2*p(1)) ;
        end
        % QA
        %figure ;
        %plot(Lat,Uh,'-',Lat(Im),m,'*r',Lat(Im-n:Im+n),polyval(p,Lat(Im-n:Im+n)),'-b',Lat(Im+n):0.01:Lat(Im-n),polyval(p,Lat(Im+n):0.01:Lat(Im-n)),'-g') ;  hold on
        %plot(PhiSH,max(polyval(p,Lat(Im+n):0.01:Lat(Im-n))),'xb') ;  hold on
    otherwise
        disp(['TropD_Metric_EDJ: ERROR: unrecognized method ''',method,''''])
        PhiNH = [] ;
        PhiSH = [] ;
end

% QA
% figure ;
% plot(lat,u,'-k','linewidth',2) ; hold on
% plot([PhiNH  PhiNH],[min(u)  max(u)],'--')
% plot([PhiSH  PhiSH],[min(u)  max(u)],'--')
% xlabel('Latitude','fontsize',14) ;
% ylabel('850hPa zonal wind','fontsize',14)
% xlim([-90 90])