% TropD PSI metric
% Latitude of the meridional mass streamfunction subtropical zero crossing
% Written by Ori Adam Mar.20.2017
% Methods:
%  'Psi_500'{default}: Most equatorward zero crossing of the stream function (Psi) at the 500hPa level, poleward of the subtropical extremum of Psi
%  'Psi_500_10Perc': Most equatorward crossing of 10% of the extremum value of Psi in each hemisphre at the 500hPa level, poleward of the subtropical extremum of Psi
%  'Psi_Levels' : Most equatorward zero crossing of Psi vertically averaged between specified levels (default values are the 300hPa and 700 hPa levels), poleward of the subtropical extremum of Psi
%  'Psi_500_Int': Most equatorward zero crossing of the vertically-integrated Psi at the 500 hPa level, , poleward of the subtropical extremum of Psi
%  'Psi_Int'    : Zero crossing of the column-intergated Psi, poleward of the subtropical extremum of Psi
%
% Syntax:
% >> [PhiSH PhiNH] = TropD_Metric_PSI(V,lat,lev,method,Lat_Uncertainty)
% Input:
% V(lat,lev) = zonal-mean meridional wind
% lat = equally-spaced latitude vector
% lev = vertical pressure-level vector [hPa]
% method (optional) = 'Psi_500' {default}|  'Psi_500_10Perc'  |  'Psi_Levels' |  'Psi_500_Int'  |  'Psi_Int'
% Lat_Uncertainty (optional, requires specifying the method): [Degrees latitude, default = 0]
%                    The minimal distance allowed between the first and second zero crossings of the same sign change
%                    For example, for Lat_Uncertainty = 10, the function will return a NaN value if a second zero crossings 
%                    is found within 10 degrees of the most equatorward zero crossing.
% Output:
% PhiSH = PSI in the SH
% PhiNH = PSI in the NH


function [PhiSH , PhiNH] = TropD_Metric_PSI(V,lat,lev,method,Lat_Uncertainty,Levels)

if nargin < 4
    method = 'Psi_500' ;
elseif nargin>4 && Lat_Uncertainty < 0
    disp 'TropD_Metric_PSI: ERROR : Lat_Uncertainty must be greater than or equal to zero'
    PhiNH = [] ;
    PhiSH = [] ;
    return
end
if nargin < 5
    Lat_Uncertainty = 0 ;
end
subpolar_boundary = 30 ;
polar_boundary = 60 ;
Psi = TropD_Calculate_StreamFunction(V,lat,lev) ;
lev = double(lev) ;
if ~iscolumn(lev)
    lev = lev' ;
end
if ~iscolumn(lat)
    lat = lat' ;
end
% Make sure lat is a monotonically increasing function
if lat(end) < lat(1)
    Psi = flip(Psi,1) ;
    lat = flip(lat)   ;
end
COS = repmat(cosd(lat),[1 length(lev)]) ;
Psi(isnan(Psi)) = 0 ; % remove nan values
switch method
    case {'Psi_500','Psi_500_10Perc'}
        % Use Psi at the level nearest to 500 hPa
        try
            P = Psi(:,dsearchn(lev,500)) ;
        catch
            P = Psi(:,lev==500) ;
        end
    case 'Psi_Levels'
        if nargin == 6
            try
                if length(Levels) > 2
                    disp(['TropD_Metric_PSI: ERROR : Levels must be a scalar or a two-element vector  '])
                    PhiNH = [] ;
                    PhiSH = [] ;
                    return
                end
            catch
                disp(['TropD_Metric_PSI: ERROR : Levels must be a scalar or a two-element vector  '])
                PhiNH = [] ;
                PhiSH = [] ;
                return
            end
            Lev1 = max(Levels) ;
            Lev2 = min(Levels) ;
        else
            Lev1 = 700 ;
            Lev2 = 300 ;
        end
        % Use Psi averaged between Lev1 and Lev2
        if isempty(lev(lev<=Lev1 & lev >= Lev2))
            disp(['TropD_Metric_PSI: ERROR : the levels specified in the input variable Levels are outside the domain of lev  '])
                    PhiNH = [] ;
                    PhiSH = [] ;
                    return
        elseif length(lev(lev<=Lev1 & lev >= Lev2))== 1
            try
                P = Psi(:,dsearchn(lev,Lev1)) ;
            catch
                P = Psi(:,lev==Lev1) ;
            end  
        else
            if lev(1) < lev(end)
                P = trapz(double(lev(lev<=Lev1 & lev >= Lev2))*100,Psi(:,lev<=Lev1 & lev >= Lev2).*COS(:,lev<=Lev1 & lev >= Lev2),2) ;
            else
                P = -trapz(double(lev(lev<=Lev1 & lev >= Lev2))*100,Psi(:,lev<=Lev1 & lev >= Lev2).*COS(:,lev<=Lev1 & lev >= Lev2),2) ;
            end
        end
    case 'Psi_500_Int'
        % Use integrated Psi from p=0 to level mearest to 500 hPa
        if lev(1) < lev(end)
            PPsi = cumtrapz(double(lev),Psi.*COS,2) ;
        else
            PPsi = flip(cumtrapz(flip(lev,1),flip(Psi,2).*COS,2),2) ;
        end
        try
            P = PPsi(:,dsearchn(lev,500)) ;
        catch
            P = PPsi(:,lev==500) ;
        end
    case 'Psi_Int'
        % Use vertical mean of Psi
        if lev(1) < lev(end)
            P = trapz(double(lev),Psi.*COS,2) ;
        else
            P = flip(trapz(flip(lev,1),flip(Psi,2).*COS,2),2) ;
        end
    otherwise
        disp(['TropD_Metric_PSI: ERROR : Unrecognized method  ''',method,''''])
        PhiNH = [] ;
        PhiSH = [] ;
        return
end

% 1. Find latitude of maximum (minimum) of Psi in the NH (SH) at the tropics (equatorward of subpolar_boundary)
% 2. Find latitude of minimum (maximum) of Psi in the NH (SH) at the extratropics (equatorward of polar_boundary)
% 3. Find the zero crossing between the above latitudes

% NH
Lmax = TropD_Calculate_MaxLat(P(lat>0 & lat < subpolar_boundary),lat(lat>0 & lat < subpolar_boundary)) ;
Lmin = TropD_Calculate_MaxLat(-P(lat>Lmax & lat < polar_boundary),lat(lat>Lmax & lat < polar_boundary)) ;
switch method
    case 'Psi_500_10Perc'
        Pmax = max(P(lat>0 & lat < subpolar_boundary)) ;
        PhiNH = TropD_Calculate_ZeroCrossing(P(lat>Lmax & lat < Lmin)- 0.1*Pmax,lat(lat>Lmax & lat < Lmin)) ;
    otherwise
        PhiNH = TropD_Calculate_ZeroCrossing(P(lat>Lmax & lat < Lmin),lat(lat>Lmax & lat < Lmin),Lat_Uncertainty) ;
end
% SH
Lmax = TropD_Calculate_MaxLat(-P(lat<0 & lat > -subpolar_boundary),lat(lat<0 & lat > -subpolar_boundary)) ;
Lmin = TropD_Calculate_MaxLat(P(lat<Lmax & lat > -polar_boundary),lat(lat<Lmax & lat > -polar_boundary)) ;
switch method
    case 'Psi_500_10Perc'
        Pmin = min(P(lat<0 & lat > -subpolar_boundary)) ;
        PhiSH = TropD_Calculate_ZeroCrossing(flip(P(lat<Lmax & lat > Lmin)) - 0.1*Pmin,flip(lat(lat<Lmax & lat > Lmin))) ; % flip in order to find the most equatorward zero crossing
    otherwise
        PhiSH = TropD_Calculate_ZeroCrossing(flip(P(lat<Lmax & lat > Lmin)),flip(lat(lat<Lmax & lat > Lmin)),Lat_Uncertainty) ; % flip in order to find the most equatorward zero crossing
end

% QA
% figure ;
% subplot (3,1,[1 2]) ;
% contourf(lat,lev,Psi') ; colormap(cool)
% axis ij ; hold on
% contour(lat,lev,Psi',[0 0],'linewidth',2,'color',[0 0 0]) ;
% plot([PhiNH  PhiNH],[100  1000],'--r')
% plot([PhiSH  PhiSH],[100  1000],'--r')
% xlim([-90 90])
% subplot 313
% plot([-90 90],[0 0],'-k','linewidth',1) ; hold on
% plot(lat,P,'-k','linewidth',3) ; hold on
% plot([PhiNH  PhiNH],[min(P)  max(P)],'--r')
% plot([PhiSH  PhiSH],[min(P)  max(P)],'--r')
% xlabel('Latitude','fontsize',14) ;
% ylabel('\Psi_{500}','fontsize',14)
% title(['Streamfunction zero crossing using the ''',regexprep(method,'_','-'),''' method'])
% xlim([-90 90])

