% TropD sea-level pressure (PSL) metric
% Latitude of maximum of the subtropical sea-level pressure
% Written by Ori Adam Mar.17.2017
% Methods:
% 'peak' {default}: the latitude of the maximum of sea-level pressure (smoothing parameter n=30)
% 'max' : the latitude of the maximum of sea-level pressure (smoothing parameter n=6)
%
% Syntax:
% >> [PhiSH PhiNH] = TropD_Metric_PSL(ps,lat,method)
% Input:
% ps(lat) = sea-level pressure
% lat = equally-spaced latitude column vector
% method (optional) = | 'peak' {default} | 'max'
% Output:
% PhiSH = latitude of subtropical sea-level pressure maximum in the SH
% PhiNH = latitude of subtropical sea-level pressure maximum in the NH

function [PhiSH , PhiNH] = TropD_Metric_PSL(ps,lat,method,n)

if nargin < 3
    method = 'peak' ;
end
if nargin == 4
    if ~isscalar(n) || n<1
        disp('TropD_Metric_PSL: ERROR : the smoothing parameter n must be >= 1' ) ;
        PhiNH = [] ;
        PhiSH = [] ;
        return
    end
else
    n = 0 ; 
end
eq_boundary    = 15 ;
polar_boundary = 60 ;
if ~iscolumn(ps)
    ps = ps' ; 
end
if ~iscolumn(lat)
    lat = lat' ; 
end
switch method
    case 'max'
        if n
        PhiNH = TropD_Calculate_MaxLat(ps(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary),n) ;
        PhiSH = TropD_Calculate_MaxLat(ps(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary),n) ;
        else
        PhiNH = TropD_Calculate_MaxLat(ps(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary)) ; % default, n=6, moderate smoothing
        PhiSH = TropD_Calculate_MaxLat(ps(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary)) ; % default, n=6, moderate smoothing
        end
    case 'peak'
        if n
        PhiNH = TropD_Calculate_MaxLat(ps(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary),n) ; 
        PhiSH = TropD_Calculate_MaxLat(ps(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary),n) ;    
        else
        PhiNH = TropD_Calculate_MaxLat(ps(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary),30) ; % for n>=30, result converges to max(F)
        PhiSH = TropD_Calculate_MaxLat(ps(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary),30) ;    
        end
    otherwise
        disp(['TropD_Metric_PSL: ERROR: unrecognized method ''',method,''''])
        PhiNH = [] ;
        PhiSH = [] ;
        return
end

% QA
%figure ;
%plot(lat,ps,'-k','linewidth',2) ; hold on
%plot([PhiNH  PhiNH],[min(ps)  max(ps)],'--')
%plot([PhiSH  PhiSH],[min(ps)  max(ps)],'--')
%xlabel('Latitude','fontsize',14) ;
%ylabel('Surface Pressure','fontsize',14)
