% TropD Subtropical Jet (STJ) metric
% Written by Ori Adam Mar.20.2017
% Methods:
%  'adjusted_peak' {Default}: Latitude of the maximum (smoothing parameter n=30) of the zonal wind averaged between the 100 and 400 hPa levels minus the zonal mean zonal wind at the level closest to the 850 hPa level,
%                        poleward of 10 degrees and equatorward of the Eddy Driven Jet latitude
%  'adjusted_max': Latitude of the maximum (smoothing parameter n=6) of the zonal wind averaged between the 100 and 400 hPa levels minus the zonal mean zonal wind at the level closest to the 850 hPa level,
%                   poleward of 10 degrees and equatorward of the Eddy Driven Jet latitude
%  'core_peak': Latitude of the maximum of the zonal wind (smoothing parameter n=30) averaged between the 100 and 400 hPa levels,
%          poleward of 10 degrees and equatorward of 60 degrees
%  'core_max': Latitude of maximum of the zonal wind (smoothing parameter n=6) averaged between the 100 and 400 hPa levels,
%               poleward of 10 degrees and equatorward of 60 degrees
%
% Syntax:
% >> [PhiSH PhiNH] = TropD_Metric_STJ(U,lat,lev,method)
% Input:
% U(lat,lev) = zonal mean zonal wind
% lat = equally-spaced latitude column vector
% lev = vertical pressure-level vector [hPa]
% method (optional) = 'adjusted_peak' {default}| 'core_peak' | 'adjusted_max' | 'core_max'
% Output:
% PhiSH = latitude of STJ in the SH
% PhiNH = latitude of STJ in the NH


function [PhiSH , PhiNH] = TropD_Metric_STJ(U,lat,lev,method,n)

if nargin < 4
    method = 'adjusted_peak' ;
end
if nargin == 5
    if ~isscalar(n) || n<1
        disp('TropD_Metric_STJ: ERROR : the smoothing parameter n must be >= 1' ) ;
        PhiNH = [] ;
        PhiSH = [] ;
        return
    end
else
    n = 0 ; 
end
eq_boundary    = 10 ;
polar_boundary = 60 ;
lev = double(lev) ;
if ~iscolumn(lev)
    lev = lev' ;
end
lev_int = lev(lev>=100 & lev<=400) ;

switch method
    case {'adjusted_peak','adjusted_max'}
        try
            idx_850  = dsearchn(lev,850) ;
        catch % in case dsearchn is not supported
            idx_850  = find(lev==850) ;
        end
        % Pressure weighted vertical mean of U minus near surface U
        if length(lev_int) > 1
            u = trapz(lev_int,U(:,lev>=100 & lev<=400),2)/(lev_int(end)-lev_int(1)) - U(:,idx_850) ;
        else
            u = mean(U(:,lev>=100 & lev<=400),2) - U(:,idx_850) ;
        end
    case {'core_peak','core_max'}
        % Pressure weighted vertical mean of U
        if length(lev_int) > 1
            u = trapz(lev_int,U(:,lev>=100 & lev<=400),2)/(lev_int(end)-lev_int(1)) ;
        else
            u = mean(U(:,lev>=100 & lev<=400),2)  ;
        end
    otherwise
        disp(['TropD_Metric_STJ: unrecognized method ''',method,''''])
        disp(['TropD_Metric_STJ: optional methods are: adjusted_peak {default}, adjusted_max, core_peak, core_max'])
        PhiNH = [] ;
        PhiSH = [] ;
        return
end


switch method
    case 'core_peak'
        if n
            PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary),n) ;
            PhiSH = TropD_Calculate_MaxLat(u(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary),n) ;
        else
            PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary),30) ;
            PhiSH = TropD_Calculate_MaxLat(u(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary),30) ;
        end
    case 'core_max'
        if n
            PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary),n) ;
            PhiSH = TropD_Calculate_MaxLat(u(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary),n) ;
        else
            PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary),6) ;
            PhiSH = TropD_Calculate_MaxLat(u(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary),6) ;
        end
    case 'adjusted_peak'
        [PhiSH_EDJ , PhiNH_EDJ] = TropD_Metric_EDJ(U,lat,lev) ;
        if n
            PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < PhiNH_EDJ),lat(lat>eq_boundary & lat < PhiNH_EDJ),n) ;
            PhiSH = TropD_Calculate_MaxLat(u(lat>PhiSH_EDJ & lat < -eq_boundary),lat(lat>PhiSH_EDJ & lat < -eq_boundary),n) ;
        else
            PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < PhiNH_EDJ),lat(lat>eq_boundary & lat < PhiNH_EDJ),30) ;
            PhiSH = TropD_Calculate_MaxLat(u(lat>PhiSH_EDJ & lat < -eq_boundary),lat(lat>PhiSH_EDJ & lat < -eq_boundary),30) ;
        end
    case 'adjusted_max'
        [PhiSH_EDJ , PhiNH_EDJ] = TropD_Metric_EDJ(U,lat,lev) ;
        if n
            PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < PhiNH_EDJ),lat(lat>eq_boundary & lat < PhiNH_EDJ),n) ;
            PhiSH = TropD_Calculate_MaxLat(u(lat>PhiSH_EDJ & lat < -eq_boundary),lat(lat>PhiSH_EDJ & lat < -eq_boundary),n) ;
        else
            PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < PhiNH_EDJ),lat(lat>eq_boundary & lat < PhiNH_EDJ),6) ;
            PhiSH = TropD_Calculate_MaxLat(u(lat>PhiSH_EDJ & lat < -eq_boundary),lat(lat>PhiSH_EDJ & lat < -eq_boundary),6) ;
        end
end

% QA
% figure ;
% subplot (3,1,[1 2]) ;
% contourf(lat,lev,U',[-40:5:100]) ;
% axis ij ; hold on
% contour(lat,lev,U',[0 0],'linewidth',2,'color',[0 0 0]) ;
% plot([PhiNH  PhiNH],[100  400],'--k')
% plot([PhiSH  PhiSH],[100  400],'--k')
% xlim([-90 90])
% subplot 313
% plot(lat,u,'-k','linewidth',2) ; hold on
% switch method
%     case {'adjusted','adjusted_peak'}
%         plot(lat,u + U(:,idx_850),'--k','linewidth',2) ; hold on
% end
% plot([PhiNH  PhiNH],[min(u)  max(u)],'--k')
% plot([PhiSH  PhiSH],[min(u)  max(u)],'--k')
% xlabel('Latitude','fontsize',14) ;
% ylabel('[100 400] zonal wind','fontsize',14)
% switch method
%     case {'adjusted','adjusted_peak'}
%         l = legend('Adjusted zonal wind','Zonal wind') ; set(l,'box','off')
%     otherwise
%         l = legend('Zonal wind') ; set(l,'box','off')
% end
%
% xlim([-90 90])