% TropD near-surface zonal wind metric
% Written by Ori Adam Mar.21.2017
% Methods:
%  'zero_crossing': the first subtropical latitude where near-surface zonal wind changes from negative to positive
% Syntax:
% >> [PhiSH PhiNH] = TropD_Metric_UAS(U,lat)
% or
% >> [PhiSH PhiNH] = TropD_Metric_UAS(U,lat,method,Lat_Uncertainty)
% Input:
% U(lat) = zonal mean near-surface zonal wind
% lat = equally-spaced latitude column vector
% method (optional) = 'zero_crossing' {default}
% Lat_Uncertainty (optional) = [Degrees latitude, default = 0] the minimal distance allowed between the first and second zero crossings of the same sign change along lat 
% Output:
% PhiSH = latitude of first subtropical zero crossing in the SH
% PhiNH = latitude of first subtropical zero crossing in the NH


function [PhiSH , PhiNH] = TropD_Metric_UAS(U,lat,method,Lat_Uncertainty)

if nargin < 3
    method = 'zero_crossing' ;
end
if nargin < 4
    Lat_Uncertainty = 0 ;
end
% U = single-level zonal wind
if min(size(U))>1
   disp(['TropD_Metric_UAS: ERROR : U must be a vector '])
        PhiNH = [] ;
        PhiSH = [] ;
        return 
end
% make lat monotonically increasing
if lat(end) < lat(1)
    U = flip(U) ;
    lat = flip(lat)   ;
end
if ~iscolumn(lat)
    lat = lat'   ;
end
if ~iscolumn(U)
    U = U'   ;
end
eq_boundary       = 5  ;
subpolar_boundary = 30 ;
polar_boundary    = 60 ;
% NH
uas_min_lat_NH = TropD_Calculate_MaxLat(-U(lat>eq_boundary & lat < subpolar_boundary),lat(lat>eq_boundary & lat < subpolar_boundary)) ;
% SH
uas_min_lat_SH = TropD_Calculate_MaxLat(-U(lat>-subpolar_boundary & lat < -eq_boundary),lat(lat>-subpolar_boundary & lat < -eq_boundary)) ;
switch method
    case 'zero_crossing'
        PhiNH          = TropD_Calculate_ZeroCrossing(U(lat>uas_min_lat_NH & lat < polar_boundary),lat(lat>uas_min_lat_NH & lat < polar_boundary),Lat_Uncertainty) ;
        PhiSH          = TropD_Calculate_ZeroCrossing(flip(U(lat<uas_min_lat_SH & lat > -polar_boundary)),flip(lat(lat<uas_min_lat_SH & lat > -polar_boundary)),Lat_Uncertainty) ; % flip in order to find the most equatorward zero crossing
    otherwise
        disp(['TropD_Metric_UAS: ERROR : unrecognized method ''',method,''''])
        PhiNH = [] ;
        PhiSH = [] ;
        return
end

% QA
%  figure ;
%  plot(lat,U*0,'-k','linewidth',1) ; hold on
%  plot(lat,U,'-k','linewidth',2) ;
%  plot([PhiNH  PhiNH],[max(U)  min(U)],'--k')
%  plot([PhiSH  PhiSH],[min(U)  max(U)],'--k')
%  xlabel('Latitude','fontsize',14) ;
%  ylabel('Near-surface zonal wind','fontsize',14)

